//package BlindControl;

import java.io.*;
import java.util.*;

//import BlindControl.messages.ParamMsg;
//import BlindControl.messages.SensorsMsg;
//import BlindControl.messages.HallMsg;
import net.tinyos.message.Message;
import net.tinyos.message.MessageListener;
import net.tinyos.message.MoteIF;
import net.tinyos.packet.BuildSource;
import net.tinyos.packet.PhoenixSource;
import net.tinyos.util.PrintStreamMessenger;

public class BlindControl implements MessageListener, Runnable{
	public static BlindControl instance = null;	// Instancia de esta clase
	MoteIF mif;					// Declaramos un objeto MoteIF para recibir mensajes...
	MoteIF mif2;				// ...y otro para enviar mensajes a la mota
	Date now;					// Marca de tiempo para entradas en log
	static File log;			// Archivo de texto log
	static DataOutputStream dos;// Filtro para flujo bytes con fichero log
	// Definiciones para CommandMsg, SensorMsg y HallMsg
	static final int NO_COMMAND 		= 0;	
	static final int COMM_MOTOR_OPEN 	= 1;
	static final int COMM_MOTOR_CLOSE 	= 2;
	static final int BLIND_CLOSED 		= 3;		
	static final int BLIND_NOT_CLOSED 	= 4;
	
	public static void main (String args[]) throws Exception{
		instance = new BlindControl();
		log = new File("Blind_Control_Log.txt");
		dos = new DataOutputStream(new FileOutputStream(log, true));
		PhoenixSource phoenix;
		String source = "serial@/dev/ttyUSB0:57600";
		phoenix = BuildSource.makePhoenix(source, PrintStreamMessenger.err);
		instance.mif = new MoteIF(phoenix);
		instance.mif2 = new MoteIF(phoenix);	
		instance.mif.registerListener(new SensorsMsg(), instance);
		instance.mif.registerListener(new HallMsg(), instance);	
		instance.mif.registerListener(new ParamMsg(), instance);	// No necesaria???
		// Nuevo hilo para gestionar interface
		Thread userInterface = new Thread(instance, "Interface");
		userInterface.start();		
	}
				
/* Recepción de mensajes y su registro en log *********************************/				
	@Override
	public void messageReceived(int to, Message m) {
		now = new Date();
		if(m instanceof SensorsMsg){
			SensorsMsg sm = (SensorsMsg) m;
			printLog(now.toString()+" - COUNTER: "+sm.get_counter()+"\tC_PHOTO: "+sm.get_countsPhoto()+"\tTEMP: "+sm.get_countsTemp()/15+"ºC"+"\tmoteId: "+sm.get_moteId()+'\r'+'\n');
			if(sm.get_commnd() == NO_COMMAND){
				printLog("                               ");
				printLog("COMMAND: No hay comando"+'\r'+'\n');
			}
			else if(sm.get_commnd() == COMM_MOTOR_OPEN){
				printLog("                               ");
				printLog("COMMAND: Abrir persiana"+"\tTIEMPO: "+sm.get_timeCmd()+'\r'+'\n');
			}
			else if(sm.get_commnd() == COMM_MOTOR_CLOSE){
				printLog("                               ");
				printLog("COMMAND: Cerrar persiana"+"\tTIEMPO: "+sm.get_timeCmd()+'\r'+'\n');
			}
		}
		else if(m instanceof HallMsg){
			HallMsg ham = (HallMsg) m; 
			printLog(now.toString()+" - HALL EVENT número: "+ham.get_eventNumber());
			if(ham.get_blindState() == BLIND_CLOSED){
				printLog(":  Persiana cerrada"+'\r'+'\n');
			}
			if(ham.get_blindState() == BLIND_NOT_CLOSED){
				printLog(" Persiana no cerrada"+'\r'+'\n');
			}
		}
		else{
			System.err.println("  Error:  mensaje inesperado con amType "+m.amType());
		}
	}
	
/* Hilo de interface y de envío de parámetros *********************************/	
	BufferedReader keyb = new BufferedReader(new InputStreamReader(System.in));
	String entryString;						// Cadena auxiliar (entrada teclado)
	int tMin = 20, tMax = 24, freq = 600;	// Parámetros sistema por defecto
	String editor = "gedit";				// Editor/visor log por defecto
	
	@Override
	public void run() {
		paramEnter();		// Primer uso:  entramos parámetros iniciales,...
		sendParam();		// ...los enviamos a la mota...
		while(true){		// ...e iniciamos bucle permanente de la interfaz de usuario
			mainScreen();
		}
	}

	private void mainScreen(){
		println("");
		println("");
		println("-------------------------------------------");
		println("C O N T R O L   P E R S I A N A   M O V I L");
		println("-------------------------------------------");
		println("");
		println("           == MENU PRINCIPAL ==            ");
		println("");
		println("");
		println("PARAMETROS ACTUALES:");
		println("");
		println("    Temperatura mínima:  "+tMin+"º C");
		println("    Temperatura máxima:  "+tMax+"º C");
		println("    Frecuencia muestreo: "+freq+" minutos");
		println("");
		println("    Visor del log: "+editor);		
		println("");
		println("OPCIONES DEL PROGRAMA:");
		//println("");
		println("    1 - Revisar log");
		println("    2 - Modificar parámetros");
		println("    3 - Cambiar editor para revisar log");
		println("");
		println("");
		println("> Introduzca opción [1, 2 o 3]:  ");
		try {		
			entryString = keyb.readLine();
		} catch (IOException e) {
			System.err.println("  Error:  no se ha podido leer del teclado");
		}	
		if (entryString.equals("1")){
			logReader();
		}
		else if (entryString.equals("2")){
			paramEnter();
			sendParam();
		}
		else if (entryString.equals("3")){
			editorEnter();
		}
		else{
			println("  Error:  opción no válida.  Teclee 1, 2 o 3");
		}			
	}

	private void logReader(){
		Runtime rt = Runtime.getRuntime();
		try{
			Process pr = rt.exec(editor+" Blind_Control_Log.txt");
		} catch(IOException e){
			System.err.println("  Error:  no se ha podido arrancar el visor del log");
			System.err.println("          Comprobar nombre y/o acceso al editor");
		}
		println("  --Log abierto con "+editor);		
		println("> Pulse intro para continuar");
		try {		
			keyb.readLine();
		} catch (IOException e) {
			System.err.println("  Error:  no se ha podido leer del teclado");
		}			
	}

	private void paramEnter(){
		println("");
		println("-------------------------------------------");		
		println("C O N T R O L   P E R S I A N A   M O V I L");
		println("-------------------------------------------");
		println("     == INTRODUCCION DE PARAMETROS ==      ");
		println("");
		println("> Introduzca temperatura mínima (en º C):  ");
		try {		
			entryString = keyb.readLine();
		} catch (IOException e) {
			System.err.println("  Error:  no se ha podido leer del teclado");
		}	
		tMin = Integer.parseInt(entryString);
		println("> Introduzca temperatura máxima (en º C):  ");
		try {		
			entryString = keyb.readLine();
		} catch (IOException e) {
			System.err.println("  Error:  no se ha podido leer del teclado");
		}	
		tMax = Integer.parseInt(entryString);
		println("> Introduzca frecuencia muestreo (en minutos):  ");
		try {		
			entryString = keyb.readLine();
		} catch (IOException e) {
			System.err.println("  Error:  no se ha podido leer del teclado");
		}	
		freq = Integer.parseInt(entryString);		
		println("  --Introducción de parámetros finalizada");
		// Registramos cambio de parámetros en log
		now = new Date();
		printLog(now.toString()+" - Nuevos parámetros:  tempMax = "+tMax+"; tempMin = "+tMin+"; freq = "+freq+'\r'+'\n');
	}
	
	private void sendParam(){
		println("  --Enviando mensaje a nodo control con los nuevos parámetros...");
		ParamMsg msg = new ParamMsg();
		msg.set_tMin((short)tMin);
		msg.set_tMax((short)tMax);
		msg.set_freqOp(freq);
		try {		
			mif2.send(1, msg);
		} catch (IOException e) {
			System.err.println("  Error:  no se ha podido enviar mensaje al nodo");
		}	
		println("  --Nuevos parámetros del sistema fijados");		
		println("> Pulse intro para continuar");
		try {		
			keyb.readLine();
		} catch (IOException e) {
			System.err.println("  Error:  no se ha podido leer del teclado");
		}	
	}
	
	private void editorEnter(){
		println("");
		println("-------------------------------------------");		
		println("C O N T R O L   P E R S I A N A   M O V I L");
		println("-------------------------------------------");
		println("       == INTRODUCCION DE EDITOR ==        ");
		println("");
		println("> Introduzca nombre editor (nombre del ejecutable):  ");
		try {		
			editor = keyb.readLine();
		} catch (IOException e) {
			System.err.println("  Error:  no se ha podido leer del teclado");
		}		
		println("  --Nuevo editor del sistema fijado:  "+editor);		
		println("> Pulse intro para continuar");
		try {		
			keyb.readLine();
		} catch (IOException e) {
			System.err.println("  Error:  no se ha podido leer del teclado");
		}			
	}		

/* Auxiliares *****************************************************************/	
	private static void printLog(String s){
		try{
			dos.writeBytes(s);
		} catch(Exception e){
			System.err.println("  Error:  no se ha podido escribir en el log");
		} 
	}

	private static void println(String s){
		System.out.println(s);
	}
	
	private static void print(String s){
		System.out.print(s);
	}
		
}
